@@ -32,8 +32,14 @@
 #include "ctkconfig.h"
 #include "ctkhelp.h"
 #include "ctkditheringcontrols.h"
+#include "ctkdropdownmenu.h"
 
 /* function prototypes */
+static void
+ctk_dither_controls_class_init(CtkDitheringControlsClass *ctk_object_class);
+
+static void ctk_dither_controls_finalize(GObject *object);
+
 static gboolean build_dithering_mode_table(CtkDitheringControls *ctk_dithering_controls);
 
 static gint map_nvctrl_value_to_table(CtkDitheringControls *ctk_dithering_controls,
@@ -41,11 +47,21 @@ static gint map_nvctrl_value_to_table(CtkDitheringControls *ctk_dithering_contro
 
 static Bool update_dithering_info(gpointer user_data);
 
-static void dithering_depth_menu_changed(GtkOptionMenu *dithering_depth_menu,
+static void setup_dithering_info(CtkDitheringControls *ctk_dithering_controls);
+
+static void setup_reset_button(CtkDitheringControls *ctk_dithering_controls);
+
+static void setup_dithering_config_menu(CtkDitheringControls *ctk_dithering_controls);
+
+static void setup_dithering_mode_menu(CtkDitheringControls *ctk_dithering_controls);
+
+static void setup_dithering_depth_menu(CtkDitheringControls *ctk_dithering_controls);
+
+static void dithering_depth_menu_changed(GtkWidget *dithering_depth_menu,
                                          gpointer user_data);
-static void dithering_mode_menu_changed(GtkOptionMenu *dithering_mode_menu,
+static void dithering_mode_menu_changed(GtkWidget *dithering_mode_menu,
                                         gpointer user_data);
-static void dithering_config_menu_changed(GtkOptionMenu *dithering_config_menu,
+static void dithering_config_menu_changed(GtkWidget *dithering_config_menu,
                                           gpointer user_data);
 
 static void dithering_update_received(GtkObject *object, gpointer arg1,
@@ -63,6 +79,10 @@ static
 void post_dithering_depth_update(CtkDitheringControls *ctk_dithering_controls,
                                  gint dithering_depth);
 
+static gint map_dithering_config_menu_idx_to_nvctrl(gint idx);
+
+static gint map_dithering_depth_menu_idx_to_nvctrl(gint idx);
+
 /* macros */
 #define FRAME_PADDING 5
 
@@ -92,12 +112,13 @@ GType ctk_dithering_controls_get_type(void)
             sizeof (CtkDitheringControlsClass),
             NULL, /* base_init */
             NULL, /* base_finalize */
-            NULL, /* class_init, */
+            (GClassInitFunc) ctk_dither_controls_class_init, /* class_init, */
             NULL, /* class_finalize */
             NULL, /* class_data */
             sizeof (CtkDitheringControls),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_dithering_controls_type =
@@ -109,17 +130,45 @@ GType ctk_dithering_controls_get_type(void)
     return ctk_dithering_controls_type;
 } /* ctk_dithering_controls_get_type() */
 
+static void
+ctk_dither_controls_class_init(CtkDitheringControlsClass *ctk_object_class)
+{
+    GObjectClass *gobject_class = (GObjectClass *)ctk_object_class;
+    gobject_class->finalize = ctk_dither_controls_finalize;
+}
+
+static void ctk_dither_controls_finalize(GObject *object)
+{
+    CtkDitheringControls *ctk_object = CTK_DITHERING_CONTROLS(object);
+
+    g_signal_handlers_disconnect_matched(G_OBJECT(ctk_object->ctk_event),
+                                         G_SIGNAL_MATCH_DATA,
+                                         0,
+                                         0,
+                                         NULL,
+                                         NULL,
+                                         (gpointer) ctk_object);
+}
+
 GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
                                       CtkConfig *ctk_config,
                                       CtkEvent *ctk_event,
                                       GtkWidget *reset_button,
-                                      unsigned int display_device_mask,
                                       char *name)
 {
     GObject *object;
     CtkDitheringControls *ctk_dithering_controls;
     GtkWidget *frame, *vbox, *hbox, *label;
-    GtkWidget *menu, *table, *menu_item = NULL, *separator;
+    GtkWidget *table, *separator;
+    CtkDropDownMenu *menu;
+    ReturnStatus ret;
+    int tmp;
+
+    /* test that dithering is available before creating the widget */
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_DITHERING, &tmp);
+    if (ret != NvCtrlSuccess) {
+        return NULL;
+    }
 
     /* create the object */
     object = g_object_new(CTK_TYPE_DITHERING_CONTROLS, NULL);
@@ -129,16 +178,16 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
 
     ctk_dithering_controls = CTK_DITHERING_CONTROLS(object);
     ctk_dithering_controls->handle = handle;
+    ctk_dithering_controls->ctk_event = ctk_event;
     ctk_dithering_controls->ctk_config = ctk_config;
     ctk_dithering_controls->reset_button = reset_button;
-    ctk_dithering_controls->display_device_mask = display_device_mask;
     ctk_dithering_controls->name = strdup(name);
 
     /* create main dithering box & frame */
 
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(object), hbox, FALSE, FALSE, FRAME_PADDING);
-    ctk_dithering_controls->dithering_controls_main = hbox;
+    ctk_dithering_controls->dithering_controls_box = hbox;
 
     frame = gtk_frame_new("Dithering Controls");
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
@@ -151,24 +200,15 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
 
     /* Build Dithering widgets & pack them in table */
     /* dropdown list for dithering configuration */
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
 
-    menu_item = gtk_menu_item_new_with_label("Auto");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
+    ctk_drop_down_menu_append_item(menu, "Auto", 0);
+    ctk_drop_down_menu_append_item(menu, "Enabled", 1);
+    ctk_drop_down_menu_append_item(menu, "Disabled", 2);
 
-    menu_item = gtk_menu_item_new_with_label("Enabled");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    menu_item = gtk_menu_item_new_with_label("Disabled");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    ctk_dithering_controls->dithering_config_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_config_menu),
-         menu);
+    ctk_dithering_controls->dithering_config_menu = GTK_WIDGET(menu);
+    
     ctk_config_set_tooltip(ctk_config, 
                            ctk_dithering_controls->dithering_config_menu, 
                            __dithering_config_help);
@@ -205,7 +245,7 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
     gtk_table_attach(GTK_TABLE(table), hbox, 3, 4, 0, 1,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     label = gtk_label_new(NULL);
-    ctk_dithering_controls->dithering_current_config = label;
+    ctk_dithering_controls->dithering_config_txt = label;
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
@@ -217,7 +257,8 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
 
     /* dropdown list for dithering modes - populated in setup */
-    ctk_dithering_controls->dithering_mode_menu = gtk_option_menu_new();
+    ctk_dithering_controls->dithering_mode_menu = 
+                ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
     ctk_config_set_tooltip(ctk_config, 
                            ctk_dithering_controls->dithering_mode_menu, 
                            __dithering_mode_help);
@@ -255,7 +296,7 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
     gtk_table_attach(GTK_TABLE(table), hbox, 3, 4, 2, 3,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     label = gtk_label_new(NULL);
-    ctk_dithering_controls->dithering_current_mode = label;
+    ctk_dithering_controls->dithering_mode_txt = label;
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
@@ -267,24 +308,15 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
 
     /* dithering depth */
-    menu = gtk_menu_new();
+    menu = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
 
-    menu_item = gtk_menu_item_new_with_label("Auto");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
+    ctk_drop_down_menu_append_item(menu, "Auto", 0);
+    ctk_drop_down_menu_append_item(menu, "6 bpc", 1);
+    ctk_drop_down_menu_append_item(menu, "8 bpc", 2);
 
-    menu_item = gtk_menu_item_new_with_label("6 bpc");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    menu_item = gtk_menu_item_new_with_label("8 bpc");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    ctk_dithering_controls->dithering_depth_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_depth_menu),
-         menu);
+    ctk_dithering_controls->dithering_depth_menu = GTK_WIDGET(menu);
+    
     ctk_config_set_tooltip(ctk_config, 
                            ctk_dithering_controls->dithering_depth_menu, 
                            __dithering_depth_help);
@@ -322,7 +354,7 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
     gtk_table_attach(GTK_TABLE(table), hbox, 3, 4, 4, 5,
                      GTK_FILL, GTK_FILL | GTK_EXPAND, 5, 0);
     label = gtk_label_new(NULL);
-    ctk_dithering_controls->dithering_current_depth = label;
+    ctk_dithering_controls->dithering_depth_txt = label;
     gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
@@ -361,6 +393,185 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
 } /* ctk_dithering_controls_new() */
 
 
+
+/*
+ * setup_reset_button() - enables the reset button if any of the current
+ * settings are not the default.
+ */
+static void setup_reset_button(CtkDitheringControls *ctk_dithering_controls)
+{
+    gint history;
+    gint val;
+    CtkDropDownMenu *dithering_config_menu;
+    CtkDropDownMenu *dithering_mode_menu;
+    CtkDropDownMenu *dithering_depth_menu;
+
+    if (!GTK_WIDGET_SENSITIVE(ctk_dithering_controls->dithering_controls_box)) {
+        /* Nothing is available, don't bother enabling the reset button yet. */
+        return;
+    }
+
+    /* The config menu is always available */
+    dithering_config_menu =
+        CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_config_menu);
+    history = ctk_drop_down_menu_get_current_value(dithering_config_menu);
+    val = map_dithering_config_menu_idx_to_nvctrl(history);
+    if (val != NV_CTRL_DITHERING_AUTO) {
+        goto enable;
+    }
+
+    if (GTK_WIDGET_SENSITIVE(ctk_dithering_controls->dithering_mode_box)) {
+        dithering_mode_menu =
+            CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_mode_menu);
+        history = ctk_drop_down_menu_get_current_value(dithering_mode_menu);
+        val = ctk_dithering_controls->dithering_mode_table[history];
+        if (val != NV_CTRL_DITHERING_MODE_AUTO) {
+            goto enable;
+        }
+    }
+
+    if (GTK_WIDGET_SENSITIVE(ctk_dithering_controls->dithering_depth_box)) {
+        dithering_depth_menu =
+            CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_depth_menu);
+        history = ctk_drop_down_menu_get_current_value(dithering_depth_menu);
+        val = map_dithering_depth_menu_idx_to_nvctrl(history);
+        if (val != NV_CTRL_DITHERING_DEPTH_AUTO) {
+            goto enable;
+        }
+    }
+
+    /* Don't disable reset button here, since other settings that are not
+     * managed by the ctk_image_slider here may need it enabled
+     */
+    return;
+
+ enable:
+    gtk_widget_set_sensitive(ctk_dithering_controls->reset_button, TRUE);
+}
+
+
+
+static void
+setup_dithering_depth_menu(CtkDitheringControls *ctk_dithering_controls)
+{
+    CtkDropDownMenu *dithering_depth_menu;
+    gint val;
+    ReturnStatus ret;
+    dithering_depth_menu =
+        CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_depth_menu);
+
+    /* dithering depth */
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_DITHERING_DEPTH, &val);
+    if (ret != NvCtrlSuccess) {
+        val = NV_CTRL_DITHERING_DEPTH_AUTO;
+    }
+
+    g_signal_handlers_block_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
+         G_CALLBACK(dithering_depth_menu_changed),
+         (gpointer) ctk_dithering_controls);
+
+    ctk_drop_down_menu_set_current_value(dithering_depth_menu, val);
+
+    g_signal_handlers_unblock_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
+         G_CALLBACK(dithering_depth_menu_changed),
+         (gpointer) ctk_dithering_controls);
+}
+
+
+
+static void
+setup_dithering_mode_menu(CtkDitheringControls *ctk_dithering_controls)
+{
+    CtkDropDownMenu *dithering_mode_menu;
+    gint val, i;
+    ReturnStatus ret;
+    dithering_mode_menu =
+        CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_mode_menu);
+
+    /* setup dithering modes */
+    build_dithering_mode_table(ctk_dithering_controls);
+
+    g_signal_handlers_block_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
+         G_CALLBACK(dithering_mode_menu_changed),
+         (gpointer) ctk_dithering_controls);
+
+    /* populate dropdown list for dithering modes */
+
+    ctk_drop_down_menu_reset(dithering_mode_menu);
+
+    for (i = 0; i < ctk_dithering_controls->dithering_mode_table_size; i++) {
+        switch (ctk_dithering_controls->dithering_mode_table[i]) {
+        case NV_CTRL_DITHERING_MODE_DYNAMIC_2X2:
+            ctk_drop_down_menu_append_item(dithering_mode_menu, "Dynamic 2x2", i);
+            break;
+        case NV_CTRL_DITHERING_MODE_STATIC_2X2:
+            ctk_drop_down_menu_append_item(dithering_mode_menu, "Static 2x2", i);
+            break;
+        case NV_CTRL_DITHERING_MODE_TEMPORAL:
+            ctk_drop_down_menu_append_item(dithering_mode_menu, "Temporal", i);
+            break;
+        default:
+        case NV_CTRL_DITHERING_MODE_AUTO:
+            ctk_drop_down_menu_append_item(dithering_mode_menu, "Auto", i);
+            break;
+        }
+    }
+
+    /* dithering mode */
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_DITHERING_MODE, &val);
+    if (ret != NvCtrlSuccess) {
+        val = NV_CTRL_DITHERING_MODE_AUTO;
+    }
+
+    val = map_nvctrl_value_to_table(ctk_dithering_controls, val);
+
+    ctk_drop_down_menu_set_current_value(dithering_mode_menu, val);
+
+
+    g_signal_handlers_unblock_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
+         G_CALLBACK(dithering_mode_menu_changed),
+         (gpointer) ctk_dithering_controls);
+}
+
+
+
+static void
+setup_dithering_config_menu(CtkDitheringControls *ctk_dithering_controls)
+{
+    CtkDropDownMenu *dithering_config_menu;
+    gint val;
+    dithering_config_menu =
+        CTK_DROP_DOWN_MENU(ctk_dithering_controls->dithering_config_menu);
+
+    /* dithering */
+    if (NvCtrlSuccess !=
+        NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                           NV_CTRL_DITHERING, &val)) {
+        val = NV_CTRL_DITHERING_AUTO;
+        return;
+    }
+
+    g_signal_handlers_block_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
+         G_CALLBACK(dithering_config_menu_changed),
+         (gpointer) ctk_dithering_controls);
+
+    ctk_drop_down_menu_set_current_value(dithering_config_menu, val);
+
+    g_signal_handlers_unblock_by_func
+        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
+         G_CALLBACK(dithering_config_menu_changed),
+         (gpointer) ctk_dithering_controls);
+}
+
+
+
 /*
  * ctk_dithering_controls_setup() - Setup routine for dithering attributes. Used
  * in DFP setup stage as well as for updating the GUI when there is change in
@@ -368,231 +579,129 @@ GtkWidget* ctk_dithering_controls_new(NvCtrlAttributeHandle *handle,
  */
 void ctk_dithering_controls_setup(CtkDitheringControls *ctk_dithering_controls)
 {
-    GtkWidget *menu, *menu_item = NULL;
-    gint val, i;
 
     if (!ctk_dithering_controls) {
         return;
     }
+    
+    /* setup dithering config menu */
+    setup_dithering_config_menu(ctk_dithering_controls);
+    
+    /* setup dithering mode menu */
+    setup_dithering_mode_menu(ctk_dithering_controls);
+    
+    /* setup dithering depth menu */
+    setup_dithering_depth_menu(ctk_dithering_controls);
 
-    /* dithering */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_DITHERING, &val)) {
-        val = NV_CTRL_DITHERING_AUTO;
-    }
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
-         G_CALLBACK(dithering_config_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_config_menu),
-         val);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
-         G_CALLBACK(dithering_config_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    /* setup dithering modes */
-    build_dithering_mode_table(ctk_dithering_controls);
-
-    /* populate dropdown list for dithering modes */
-    menu = gtk_menu_new();
-    for (i = 0; i < ctk_dithering_controls->dithering_mode_table_size; i++) {
-        switch (ctk_dithering_controls->dithering_mode_table[i]) {
-        case NV_CTRL_DITHERING_MODE_DYNAMIC_2X2:
-            menu_item = gtk_menu_item_new_with_label("Dynamic 2x2");
-            break;
-        case NV_CTRL_DITHERING_MODE_STATIC_2X2:
-            menu_item = gtk_menu_item_new_with_label("Static 2x2");
-            break;
-        case NV_CTRL_DITHERING_MODE_TEMPORAL:
-            menu_item = gtk_menu_item_new_with_label("Temporal");
-            break;
-        default:
-        case NV_CTRL_DITHERING_MODE_AUTO:
-            menu_item = gtk_menu_item_new_with_label("Auto");
-            break;
-        }
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-        gtk_widget_show(menu_item);
-    }
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_mode_menu),
-         menu);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-
-    if (!update_dithering_info((gpointer)ctk_dithering_controls)) {
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_controls_main,
-                                 FALSE);
-    }
+    setup_dithering_info(ctk_dithering_controls);
 
 } /* ctk_dithering_controls_setup() */
 
+static void setup_dithering_info(CtkDitheringControls *ctk_dithering_controls)
+{
+
+    if (!update_dithering_info((gpointer)ctk_dithering_controls)) {
+        gtk_widget_hide(ctk_dithering_controls->dithering_controls_box);
+    } else {
+        gtk_widget_show(ctk_dithering_controls->dithering_controls_box);
+    }
+    setup_reset_button(ctk_dithering_controls);
+
+} /* setup_dithering_info() */
 
 static Bool update_dithering_info(gpointer user_data)
 {
     CtkDitheringControls *ctk_dithering_controls =
         CTK_DITHERING_CONTROLS(user_data);
-    gint val, dithering_mode, dithering_depth;
+    ReturnStatus ret;
+    gint val;
 
-    /* requested dithering */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_DITHERING, &val)) {
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_controls_main, 
-                                 FALSE);
-    } else if (val == NV_CTRL_DITHERING_ENABLED ||
-               val == NV_CTRL_DITHERING_AUTO) {
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_controls_main, 
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_DITHERING, &val);
+    if (ret != NvCtrlSuccess) {
+        /* Dithering is not currently available */
+        return FALSE;
+    }
+
+    if (val == NV_CTRL_DITHERING_ENABLED ||
+        val == NV_CTRL_DITHERING_AUTO) {
+        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_mode_box,
+                                 TRUE);
+        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_depth_box,
                                  TRUE);
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_mode_box, TRUE);
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_depth_box, TRUE);
-        gtk_widget_show(ctk_dithering_controls->dithering_mode_box);
-        gtk_widget_show(ctk_dithering_controls->dithering_depth_box);
     } else if (val == NV_CTRL_DITHERING_DISABLED) {
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_controls_main, 
-                                 TRUE);
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_mode_box, FALSE);
-        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_depth_box, FALSE);
+        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_mode_box,
+                                 FALSE);
+        gtk_widget_set_sensitive(ctk_dithering_controls->dithering_depth_box,
+                                 FALSE);
     }
 
     /* current dithering */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_CURRENT_DITHERING, &val)) {
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_CURRENT_DITHERING, &val);
+    if (ret != NvCtrlSuccess) {
         val = NV_CTRL_CURRENT_DITHERING_DISABLED;
     }
 
     if (val == NV_CTRL_CURRENT_DITHERING_ENABLED) {
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_config),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_config_txt),
                            "Enabled");
     } else {
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_config),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_config_txt),
                            "Disabled");
     }
 
-    /* dithering mode */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_DITHERING_MODE,
-                                  &dithering_mode)) {
-        dithering_mode = NV_CTRL_DITHERING_MODE_AUTO;
-    }
-
-    dithering_mode = map_nvctrl_value_to_table(ctk_dithering_controls,
-                                               dithering_mode);
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_mode_menu),
-         dithering_mode);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
     /* current dithering mode */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_CURRENT_DITHERING_MODE,
-                                  &dithering_mode)) {
-        dithering_mode = NV_CTRL_CURRENT_DITHERING_MODE_NONE;
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_CURRENT_DITHERING_MODE, &val);
+    if (ret != NvCtrlSuccess) {
+        val = NV_CTRL_CURRENT_DITHERING_MODE_NONE;
     }
 
-    switch (dithering_mode) {
+    switch (val) {
     case NV_CTRL_CURRENT_DITHERING_MODE_DYNAMIC_2X2:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_mode),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_mode_txt),
                            "Dynamic 2x2");
         break;
     case NV_CTRL_CURRENT_DITHERING_MODE_STATIC_2X2:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_mode),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_mode_txt),
                            "Static 2x2");
         break;
     case NV_CTRL_CURRENT_DITHERING_MODE_TEMPORAL:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_mode),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_mode_txt),
                            "Temporal");
         break;
     default:
     case NV_CTRL_CURRENT_DITHERING_MODE_NONE:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_mode),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_mode_txt),
                            "None");
         break;
     }
-
-    /* dithering depth */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_DITHERING_DEPTH,
-                                  &dithering_depth)) {
-        dithering_depth = NV_CTRL_DITHERING_DEPTH_AUTO;
-    }
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
-         G_CALLBACK(dithering_depth_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_depth_menu),
-         dithering_depth);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
-         G_CALLBACK(dithering_depth_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
     /* current dithering depth */
-    if (NvCtrlSuccess !=
-        NvCtrlGetDisplayAttribute(ctk_dithering_controls->handle,
-                                  ctk_dithering_controls->display_device_mask,
-                                  NV_CTRL_CURRENT_DITHERING_DEPTH,
-                                  &dithering_depth)) {
-        dithering_depth = NV_CTRL_CURRENT_DITHERING_DEPTH_NONE;
+    ret = NvCtrlGetAttribute(ctk_dithering_controls->handle,
+                             NV_CTRL_CURRENT_DITHERING_DEPTH, &val);
+    if (ret != NvCtrlSuccess) {
+        val = NV_CTRL_CURRENT_DITHERING_DEPTH_NONE;
     }
 
-    switch (dithering_depth) {
+    switch (val) {
     case NV_CTRL_CURRENT_DITHERING_DEPTH_6_BITS:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_depth),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_depth_txt),
                            "6 bpc");
         break;
     case NV_CTRL_CURRENT_DITHERING_DEPTH_8_BITS:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_depth),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_depth_txt),
                            "8 bpc");
         break;
     default:
     case NV_CTRL_CURRENT_DITHERING_DEPTH_NONE:
-        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_current_depth),
+        gtk_label_set_text(GTK_LABEL(ctk_dithering_controls->dithering_depth_txt),
                            "None");
         break;
     }
 
-    return True;
+    return TRUE;
+
 } /* update_dithering_info() */
 
 static
@@ -662,135 +771,71 @@ void post_dithering_depth_update(CtkDitheringControls *ctk_dithering_controls,
                                  ctk_dithering_controls->name);
 }
 
-static void dithering_config_menu_changed(GtkOptionMenu *dithering_config_menu,
+static void dithering_config_menu_changed(GtkWidget *dithering_config_menu,
                                           gpointer user_data)
 {
     CtkDitheringControls *ctk_dithering_controls =
         CTK_DITHERING_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(dithering_config_menu);
     gint history, dithering_config = NV_CTRL_DITHERING_AUTO;
 
-    history = gtk_option_menu_get_history(dithering_config_menu);
+    history = ctk_drop_down_menu_get_current_value(menu);
 
-    switch (history) {
-    case 2:
-        dithering_config = NV_CTRL_DITHERING_DISABLED;
-        break;
-    case 1:
-        dithering_config = NV_CTRL_DITHERING_ENABLED;
-        break;
-    default:
-    case 0:
-        dithering_config = NV_CTRL_DITHERING_AUTO;
-        break;
-    }
+    dithering_config = map_dithering_config_menu_idx_to_nvctrl(history);
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING,
-                              dithering_config);
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
-         G_CALLBACK(dithering_config_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_config_menu),
-         dithering_config);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_config_menu),
-         G_CALLBACK(dithering_config_menu_changed),
-         (gpointer) ctk_dithering_controls);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING,
+                       dithering_config);
 
     /* reflecting the change in configuration to other widgets & reset button */
-    ctk_dithering_controls_setup(ctk_dithering_controls);
+    setup_dithering_info(ctk_dithering_controls);
     post_dithering_config_update(ctk_dithering_controls, dithering_config);
 
 } /* dithering_config_menu_changed() */
 
 
-static void dithering_mode_menu_changed(GtkOptionMenu *dithering_mode_menu,
+static void dithering_mode_menu_changed(GtkWidget *dithering_mode_menu,
                                         gpointer user_data)
 {
     CtkDitheringControls *ctk_dithering_controls =
         CTK_DITHERING_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(dithering_mode_menu);
     gint history, dithering_mode = NV_CTRL_DITHERING_MODE_AUTO;
 
-    history = gtk_option_menu_get_history(dithering_mode_menu);
+    history = ctk_drop_down_menu_get_current_value(menu);
 
     dithering_mode = ctk_dithering_controls->dithering_mode_table[history];
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING_MODE,
-                              dithering_mode);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING_MODE,
+                       dithering_mode);
 
     dithering_mode = map_nvctrl_value_to_table(ctk_dithering_controls,
                                                dithering_mode);
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_mode_menu),
-         dithering_mode);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_mode_menu),
-         G_CALLBACK(dithering_mode_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    /* reflecting the change in mode to other widgets & reset button */
-    ctk_dithering_controls_setup(ctk_dithering_controls);
+    /* reflecting the change in mode to the reset button */
+    setup_dithering_info(ctk_dithering_controls);
     post_dithering_mode_update(ctk_dithering_controls, dithering_mode);
 
 } /* dithering_mode_menu_changed() */
 
-static void dithering_depth_menu_changed(GtkOptionMenu *dithering_depth_menu,
+static void dithering_depth_menu_changed(GtkWidget *dithering_depth_menu,
                                          gpointer user_data)
 {
     CtkDitheringControls *ctk_dithering_controls =
         CTK_DITHERING_CONTROLS(user_data);
+    CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(dithering_depth_menu);
     gint history, dithering_depth = NV_CTRL_DITHERING_DEPTH_AUTO;
 
-    history = gtk_option_menu_get_history(dithering_depth_menu);
+    history = ctk_drop_down_menu_get_current_value(menu);
 
-    switch (history) {
-    case 2:
-        dithering_depth = NV_CTRL_DITHERING_DEPTH_8_BITS;
-        break;
-    case 1:
-        dithering_depth = NV_CTRL_DITHERING_DEPTH_6_BITS;
-        break;
-    default:
-    case 0:
-        dithering_depth = NV_CTRL_DITHERING_DEPTH_AUTO;
-        break;
-    }
+    dithering_depth = map_dithering_depth_menu_idx_to_nvctrl(history);
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING_DEPTH,
-                              dithering_depth);
-
-    g_signal_handlers_block_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
-         G_CALLBACK(dithering_depth_menu_changed),
-         (gpointer) ctk_dithering_controls);
-
-    gtk_option_menu_set_history
-        (GTK_OPTION_MENU(ctk_dithering_controls->dithering_depth_menu),
-         dithering_depth);
-
-    g_signal_handlers_unblock_by_func
-        (G_OBJECT(ctk_dithering_controls->dithering_depth_menu),
-         G_CALLBACK(dithering_depth_menu_changed),
-         (gpointer) ctk_dithering_controls);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING_DEPTH,
+                       dithering_depth);
 
     /* reflecting the change in configuration to other widgets & reset button */
-    ctk_dithering_controls_setup(ctk_dithering_controls);
+    setup_dithering_info(ctk_dithering_controls);
     post_dithering_depth_update(ctk_dithering_controls, dithering_depth);
 
 } /* dithering_depth_menu_changed() */
@@ -806,22 +851,19 @@ void ctk_dithering_controls_reset(CtkDitheringControls *ctk_dithering_controls)
         return;
     }
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING,
-                              NV_CTRL_DITHERING_AUTO);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING,
+                       NV_CTRL_DITHERING_AUTO);
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING_MODE,
-                              NV_CTRL_DITHERING_MODE_AUTO);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING_MODE,
+                       NV_CTRL_DITHERING_MODE_AUTO);
 
-    NvCtrlSetDisplayAttribute(ctk_dithering_controls->handle,
-                              ctk_dithering_controls->display_device_mask,
-                              NV_CTRL_DITHERING_DEPTH,
-                              NV_CTRL_DITHERING_DEPTH_AUTO);
+    NvCtrlSetAttribute(ctk_dithering_controls->handle,
+                       NV_CTRL_DITHERING_DEPTH,
+                       NV_CTRL_DITHERING_DEPTH_AUTO);
 
-    ctk_dithering_controls_setup(ctk_dithering_controls);
+    setup_dithering_info(ctk_dithering_controls);
 } /* ctk_dithering_controls_reset() */
 
 
@@ -832,6 +874,10 @@ void add_dithering_controls_help(CtkDitheringControls *ctk_dithering_controls,
                                  GtkTextBuffer *b,
                                  GtkTextIter *i)
 {
+    if (!ctk_dithering_controls) {
+        return;
+    }
+
     ctk_help_heading(b, i, "Dithering Controls");
     ctk_help_para(b, i, __dithering_help);
 
@@ -856,12 +902,6 @@ static void dithering_update_received(GtkObject *object, gpointer arg1,
     CtkDitheringControls *ctk_object = CTK_DITHERING_CONTROLS(user_data);
     CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
 
-    /* if the event is not for this display device, return */
-
-    if (!(event_struct->display_mask & ctk_object->display_device_mask)) {
-        return;
-    }
-
     ctk_dithering_controls_setup(ctk_object);
 
     /* update status bar message */
@@ -892,12 +932,11 @@ static gboolean build_dithering_mode_table(CtkDitheringControls *ctk_dithering_c
         free(ctk_dithering_controls->dithering_mode_table);
     }
 
-    ret = 
-    NvCtrlGetValidDisplayAttributeValues(ctk_dithering_controls->handle, 
-                                         ctk_dithering_controls->display_device_mask,
-                                         NV_CTRL_DITHERING_MODE,
-                                         &valid);
-        
+    ret =
+        NvCtrlGetValidAttributeValues(ctk_dithering_controls->handle,
+                                      NV_CTRL_DITHERING_MODE,
+                                      &valid);
+
     if (ret != NvCtrlSuccess || valid.type != ATTRIBUTE_TYPE_INT_BITS) {
         /* 
          * We do not have valid information to build a mode table
@@ -954,3 +993,25 @@ static gint map_nvctrl_value_to_table(CtkDitheringControls *ctk_dithering_contro
 
     return 0;
 } /*map_nvctrl_value_to_table() */
+
+
+static gint map_dithering_config_menu_idx_to_nvctrl(gint idx)
+{
+    switch (idx) {
+    case 2: return NV_CTRL_DITHERING_DISABLED;
+    case 1: return NV_CTRL_DITHERING_ENABLED;
+    default: /* fallthrough; w/ warning? */
+    case 0: return NV_CTRL_DITHERING_AUTO;
+    }
+}
+
+
+static gint map_dithering_depth_menu_idx_to_nvctrl(gint idx)
+{
+    switch (idx) {
+    case 2: return NV_CTRL_DITHERING_DEPTH_8_BITS;
+    case 1: return NV_CTRL_DITHERING_DEPTH_6_BITS;
+    default: /* fallthrough; w/ warning? */
+    case 0: return NV_CTRL_DITHERING_DEPTH_AUTO;
+    }
+}
