@@ -49,7 +49,7 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
 static void fsaa_setting_checkbox_toggled(GtkWidget *widget,
                                           gpointer user_data);
 
-static void fsaa_setting_menu_changed(GtkWidget *widget, gpointer user_data);
+static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data);
 
 static void fsaa_setting_update_received(GtkObject *object,
                                          gpointer arg1,
@@ -62,6 +62,15 @@ static void fsaa_value_changed(GtkRange *range, gpointer user_data);
 static void fsaa_update_received(GtkObject *object,
                                  gpointer arg1, gpointer user_data);
 
+static void fxaa_checkbox_toggled(GtkWidget *widget,
+                                  gpointer user_data);
+
+static void fxaa_update_received(GtkObject *object,
+                                 gpointer arg1, gpointer user_data);
+
+static void post_fxaa_toggled(CtkMultisample *ctk_multisample, 
+                              gboolean enable);
+
 static void
 post_log_aniso_app_override_toggled(CtkMultisample *ctk_multisample,
                                     gboolean override);
@@ -96,30 +105,47 @@ static void texture_sharpening_update_received(GtkObject *object,
                                                gpointer arg1,
                                                gpointer user_data);
 
+static void update_fxaa_from_fsaa_change(CtkMultisample *ctk_multisample,
+                                         int fsaa_value);
+static void update_fsaa_from_fxaa_change(CtkMultisample *ctk_multisample,
+                                         gboolean fxaa_enabled);
+static gchar *applicationSettings[] = {
+    "Use Application Settings",
+    "Override Application Settings",
+    "Enhance Application Settings"
+};
+
 static const char *__aa_override_app_help =
 "Enable the Antialiasing \"Override Application Setting\" "
 "checkbox to make the antialiasing slider active and "
-"override any appliation antialiasing setting with the "
+"override any application antialiasing setting with the "
 "value of the slider.";
 
 static const char *__aa_menu_help =
 "The Application Antialiasing Settings Menu allows the antialiasing "
-"setting of OpenGL applications to be overriden with the value of "
+"setting of OpenGL applications to be overridden with the value of "
 "the slider.";
 
 static const char *__aa_slider_help =
-"The Antialiasing slider controls the level of antialiasing.";
+"The Antialiasing slider controls the level of antialiasing. Using "
+"antialiasing disables FXAA.";
 
 static const char *__aniso_override_app_help =
 "Enable the Anisotropic Filtering \"Override Application Setting\" "
 "checkbox to make the anisotropic filtering slider "
-"active and override any appliation anisotropic "
+"active and override any application anisotropic "
 "filtering setting with the value of the slider.";
 
 static const char *__aniso_slider_help =
 "The Anisotropic Filtering slider controls the "
 "level of automatic anisotropic texture filtering.";
 
+static const char *__fxaa_enable_help = 
+"Enable Fast Approximate Anti-Aliasing. This option is applied to "
+"OpenGL applications that are started after this option is set. Enabling "
+"FXAA disables triple buffering, antialiasing, and other antialiasing "
+"setting methods.";
+
 static const char *__texture_sharpening_help =
 "To improve image quality, select this option "
 "to sharpen textures when running OpenGL applications "
@@ -131,24 +157,28 @@ static const char *__texture_sharpening_help =
  * help
  */
 
-#define __LOG_ANISO_RANGE (1 << 2)
-#define __TEXTURE_SHARPEN (1 << 3)
-#define __FSAA            (1 << 4)
-#define __FSAA_NONE       (1 << (__FSAA + NV_CTRL_FSAA_MODE_NONE))
-#define __FSAA_2x         (1 << (__FSAA + NV_CTRL_FSAA_MODE_2x))
-#define __FSAA_2x_5t      (1 << (__FSAA + NV_CTRL_FSAA_MODE_2x_5t))
-#define __FSAA_15x15      (1 << (__FSAA + NV_CTRL_FSAA_MODE_15x15))
-#define __FSAA_2x2        (1 << (__FSAA + NV_CTRL_FSAA_MODE_2x2))
-#define __FSAA_4x         (1 << (__FSAA + NV_CTRL_FSAA_MODE_4x))
-#define __FSAA_4x_9t      (1 << (__FSAA + NV_CTRL_FSAA_MODE_4x_9t))
-#define __FSAA_8x         (1 << (__FSAA + NV_CTRL_FSAA_MODE_8x))
-#define __FSAA_16x        (1 << (__FSAA + NV_CTRL_FSAA_MODE_16x))
-#define __FSAA_8xS        (1 << (__FSAA + NV_CTRL_FSAA_MODE_8xS))
-#define __FSAA_8xQ        (1 << (__FSAA + NV_CTRL_FSAA_MODE_8xQ))
-#define __FSAA_16xS       (1 << (__FSAA + NV_CTRL_FSAA_MODE_16xS))
-#define __FSAA_16xQ       (1 << (__FSAA + NV_CTRL_FSAA_MODE_16xQ))
-#define __FSAA_32xS       (1 << (__FSAA + NV_CTRL_FSAA_MODE_32xS))
-#define __FSAA_ENHANCE    (1 << (__FSAA + NV_CTRL_FSAA_MODE_MAX +1))
+#define __FSAA_NONE       (1 << NV_CTRL_FSAA_MODE_NONE)
+#define __FSAA_2x         (1 << NV_CTRL_FSAA_MODE_2x)
+#define __FSAA_2x_5t      (1 << NV_CTRL_FSAA_MODE_2x_5t)
+#define __FSAA_15x15      (1 << NV_CTRL_FSAA_MODE_15x15)
+#define __FSAA_2x2        (1 << NV_CTRL_FSAA_MODE_2x2)
+#define __FSAA_4x         (1 << NV_CTRL_FSAA_MODE_4x)
+#define __FSAA_4x_9t      (1 << NV_CTRL_FSAA_MODE_4x_9t)
+#define __FSAA_8x         (1 << NV_CTRL_FSAA_MODE_8x)
+#define __FSAA_16x        (1 << NV_CTRL_FSAA_MODE_16x)
+#define __FSAA_8xS        (1 << NV_CTRL_FSAA_MODE_8xS)
+#define __FSAA_8xQ        (1 << NV_CTRL_FSAA_MODE_8xQ)
+#define __FSAA_16xS       (1 << NV_CTRL_FSAA_MODE_16xS)
+#define __FSAA_16xQ       (1 << NV_CTRL_FSAA_MODE_16xQ)
+#define __FSAA_32xS       (1 << NV_CTRL_FSAA_MODE_32xS)
+#define __FSAA_32x        (1 << NV_CTRL_FSAA_MODE_32x)
+#define __FSAA_64xS       (1 << NV_CTRL_FSAA_MODE_64xS)
+#define __FSAA            (1 << (NV_CTRL_FSAA_MODE_MAX + 1))
+#define __FSAA_ENHANCE    (1 << (NV_CTRL_FSAA_MODE_MAX + 2))
+#define __FXAA            (1 << (NV_CTRL_FSAA_MODE_MAX + 3))
+#define __LOG_ANISO_RANGE (1 << (NV_CTRL_FSAA_MODE_MAX + 4))
+#define __TEXTURE_SHARPEN (1 << (NV_CTRL_FSAA_MODE_MAX + 5))
+
 
 #define FRAME_PADDING 5
 
@@ -169,6 +199,7 @@ GType ctk_multisample_get_type(
             sizeof (CtkMultisample),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_multisample_type =
@@ -199,7 +230,7 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
     GtkObject *adjustment;
     gint min, max;
     
-    gint val, app_control, override, enhance, i;
+    gint val, app_control, override, enhance, mode, i;
     
     NVCTRLAttributeValidValuesRec valid;
 
@@ -232,9 +263,9 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
         
         build_fsaa_translation_table(ctk_multisample, valid);
         
-        ret = NvCtrlGetAttribute(handle, NV_CTRL_FSAA_MODE, &val);
+        ret = NvCtrlGetAttribute(handle, NV_CTRL_FSAA_MODE, &mode);
         
-        val = map_nv_ctrl_fsaa_value_to_slider(ctk_multisample, val);
+        val = map_nv_ctrl_fsaa_value_to_slider(ctk_multisample, mode);
 
         ret0 = NvCtrlGetAttribute(handle,
                                   NV_CTRL_FSAA_APPLICATION_CONTROLLED,
@@ -348,7 +379,39 @@ GtkWidget *ctk_multisample_new(NvCtrlAttributeHandle *handle,
 
             for (i = 0; i < ctk_multisample->fsaa_translation_table_size; i++)
                 ctk_multisample->active_attributes |=
-                    (1 << (__FSAA+ctk_multisample->fsaa_translation_table[i]));
+                    (1 << ctk_multisample->fsaa_translation_table[i]);
+
+            /* FXAA Option button */
+
+            check_button = gtk_check_button_new_with_label("Enable FXAA");
+
+            if (mode == NV_CTRL_FSAA_MODE_NONE) {
+                ret = NvCtrlGetAttribute(handle, NV_CTRL_FXAA, &val);
+                if (val == NV_CTRL_FXAA_ENABLE) {
+                    gtk_widget_set_sensitive(GTK_WIDGET(scale), FALSE);
+                }
+            } else {
+                val = NV_CTRL_FXAA_DISABLE;
+            }
+            gtk_widget_set_sensitive(GTK_WIDGET(check_button),
+                                     (mode == NV_CTRL_FSAA_MODE_NONE));
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), val);
+
+            g_signal_connect(G_OBJECT(check_button), "toggled",
+                             G_CALLBACK(fxaa_checkbox_toggled),
+                             (gpointer) ctk_multisample);
+
+            g_signal_connect(G_OBJECT(ctk_event),
+                             CTK_EVENT_NAME(NV_CTRL_FXAA),
+                             G_CALLBACK(fxaa_update_received),
+                             (gpointer) ctk_multisample);
+
+            ctk_config_set_tooltip(ctk_config, check_button,
+                                   __fxaa_enable_help);
+            gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
+
+            ctk_multisample->active_attributes |= __FXAA;
+            ctk_multisample->fxaa_enable_check_button = check_button;
         }
     }
     
@@ -526,6 +589,8 @@ static void build_fsaa_translation_table(CtkMultisample *ctk_multisample,
     gint i, n = 0;
     gint index_8xs = -1;
     gint index_16x = -1;
+    gint index_32x = -1;
+    gint index_32xs = -1;
     gint mask = valid.u.bits.ints;
 
     ctk_multisample->fsaa_translation_table_size = 0;
@@ -543,15 +608,17 @@ static void build_fsaa_translation_table(CtkMultisample *ctk_multisample,
 
             if (i == NV_CTRL_FSAA_MODE_8xS) index_8xs = n;
             if (i == NV_CTRL_FSAA_MODE_16x) index_16x = n;
+            if (i == NV_CTRL_FSAA_MODE_32x) index_32x = n;
+            if (i == NV_CTRL_FSAA_MODE_32xS) index_32xs = n;
             
             n++;
         }
     }
     
     /*
-     * XXX 8xS was added to the NV_CTRL_FSAA_MODE list after 16x, but
-     * should appear before it in the slider.  If both were added to
-     * the fsaa_translation_table[], then swap their positions.
+     * XXX 8xS was added to the NV_CTRL_FSAA_MODE list after 16x, but should
+     * appear before it in the slider.  Same with 32x and 32xS.  If both were
+     * added to the fsaa_translation_table[], then re-order them appropriately.
      */
 
     if ((index_8xs != -1) && (index_16x != -1)) {
@@ -560,7 +627,14 @@ static void build_fsaa_translation_table(CtkMultisample *ctk_multisample,
         ctk_multisample->fsaa_translation_table[index_16x] =
             NV_CTRL_FSAA_MODE_8xS;
     }
-    
+
+    if ((index_32x != -1) && (index_32xs != -1)) {
+        ctk_multisample->fsaa_translation_table[index_32x] =
+            NV_CTRL_FSAA_MODE_32xS;
+        ctk_multisample->fsaa_translation_table[index_32xs] =
+            NV_CTRL_FSAA_MODE_32x;
+    }
+
     ctk_multisample->fsaa_translation_table_size = n;
 
 } /* build_fsaa_translation_table() */
@@ -623,30 +697,18 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
                                            CtkEvent *ctk_event,
                                            gboolean override, gboolean enhance)
 {
-    GtkWidget *omenu;
-    GtkWidget *menu;
-    GtkWidget *menu_item;
-    gint idx;
+    CtkDropDownMenu *d;
+
+    gint idx, i;
 
     /* Create the menu */
 
-    omenu = gtk_option_menu_new();
-
-    menu = gtk_menu_new();
+    d = (CtkDropDownMenu *)
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
     
-    menu_item = gtk_menu_item_new_with_label("Use Application Settings");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    menu_item = gtk_menu_item_new_with_label("Override Application Settings");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    menu_item = gtk_menu_item_new_with_label("Enhance Application Settings");
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-    gtk_widget_show(menu_item);
-
-    /* Set the state of the menu */
+    for (i = 0; i < ARRAY_LEN(applicationSettings); i++) {
+        ctk_drop_down_menu_append_item(d, applicationSettings[i], i);
+    }
 
     if (!override) {
         idx = 0;
@@ -658,14 +720,14 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
         }
     }
 
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+    /* set the menu item */
+    ctk_drop_down_menu_set_current_value(d, idx);
 
-    gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), idx);
-
-    ctk_config_set_tooltip(ctk_multisample->ctk_config, omenu,
+    ctk_config_set_tooltip(ctk_multisample->ctk_config, d->menu,
                            __aa_menu_help);
 
-    g_signal_connect(G_OBJECT(omenu), "changed",
+    g_signal_connect(G_OBJECT(d),
+                     "changed",
                      G_CALLBACK(fsaa_setting_menu_changed),
                      (gpointer) ctk_multisample);
 
@@ -675,7 +737,7 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
                      G_CALLBACK(fsaa_setting_update_received),
                      (gpointer) ctk_multisample);
 
-    return omenu;
+    return GTK_WIDGET(d);
 
 } /* create_fsaa_setting_menu() */
 
@@ -691,9 +753,15 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
 static void post_fsaa_setting_changed(CtkMultisample *ctk_multisample,
                                       gboolean override, gboolean enhance)
 {
+    GtkWidget *fxaa_checkbox = ctk_multisample->fxaa_enable_check_button;
+    gboolean fxaa_value = 
+        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxaa_checkbox));
+
     if (ctk_multisample->fsaa_scale) {
         gtk_widget_set_sensitive
-            (GTK_WIDGET(ctk_multisample->fsaa_scale), override);
+            (GTK_WIDGET(ctk_multisample->fsaa_scale), 
+            (override &&
+            (fxaa_value == NV_CTRL_FXAA_DISABLE)));
     }
 
     ctk_config_statusbar_message(ctk_multisample->ctk_config,
@@ -737,6 +805,9 @@ static void update_fsaa_setting(CtkMultisample *ctk_multisample,
         g_signal_handlers_unblock_by_func(G_OBJECT(range),
                                           G_CALLBACK(fsaa_value_changed),
                                           (gpointer) ctk_multisample);
+
+        update_fxaa_from_fsaa_change(ctk_multisample,
+                                     NV_CTRL_FSAA_MODE_NONE);
     }
 
     post_fsaa_setting_changed(ctk_multisample, override, enhance);
@@ -771,15 +842,18 @@ static void fsaa_setting_checkbox_toggled(GtkWidget *widget,
  * the fsaa slider.
  */
 
-static void fsaa_setting_menu_changed(GtkWidget *widget, gpointer user_data)
+static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data)
 {
     CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
     gint idx;
     gboolean override;
     gboolean enhance;
-
-    idx = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
-
+    
+    CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu)->current_selected_item_widget
+        = GTK_WIDGET(object);
+    idx = ctk_drop_down_menu_get_current_value
+        (CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu)); 
+    
     /* The FSAA dropdown menu is setup this way:
      *
      * 0 == app
@@ -826,8 +900,13 @@ static void fsaa_setting_update_received(GtkObject *object,
             ret = NvCtrlGetAttribute(ctk_multisample->handle,
                                      NV_CTRL_FSAA_APPLICATION_ENHANCED,
                                      &val);
-            enhance = val;
-            idx = enhance ? 2 : 1;
+            if (ret == NvCtrlSuccess) {
+                enhance = val;
+                idx = enhance ? 2 : 1;
+            } else {
+                enhance = FALSE;
+                idx = 0;
+            }
         } else {
             idx = 1;
         }
@@ -839,7 +918,11 @@ static void fsaa_setting_update_received(GtkObject *object,
         ret = NvCtrlGetAttribute(ctk_multisample->handle,
                                  NV_CTRL_FSAA_APPLICATION_CONTROLLED,
                                  &val);
-        override = !val; /* = !app_controlled */
+        if (ret == NvCtrlSuccess) {
+            override = !val; /* = !app_controlled */
+        } else {
+            override = FALSE;
+        }
 
         if (override) {
             idx = enhance ? 2 : 1;
@@ -858,13 +941,16 @@ static void fsaa_setting_update_received(GtkObject *object,
         GtkWidget *menu = ctk_multisample->fsaa_menu;
 
         g_signal_handlers_block_by_func
-            (G_OBJECT(menu), G_CALLBACK(fsaa_setting_menu_changed),
+            (G_OBJECT(menu),
+             G_CALLBACK(fsaa_setting_menu_changed),
              (gpointer) ctk_multisample);
-        
-        gtk_option_menu_set_history(GTK_OPTION_MENU(menu), idx);
+
+        ctk_drop_down_menu_set_current_value
+            (CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu), idx);
         
         g_signal_handlers_unblock_by_func
-            (G_OBJECT(menu), G_CALLBACK(fsaa_setting_menu_changed),
+            (G_OBJECT(menu),
+             G_CALLBACK(fsaa_setting_menu_changed),
              (gpointer) ctk_multisample);
     } else {
         /* Update the checkbox */
@@ -904,6 +990,87 @@ static void post_fsaa_value_changed(CtkMultisample *ctk_multisample, gint val)
 } /* post_fsaa_value_changed() */
 
 
+/*
+ * update_fxaa_from_fsaa_change - helper function for changes to fsaa in order
+ * to update fxaa and enable/disable fxaa or fsaa widgets based on the new 
+ * value of fsaa.
+ */
+
+static void update_fxaa_from_fsaa_change(CtkMultisample *ctk_multisample,
+                                         int fsaa_value)
+{
+    GtkRange *fsaa_range = GTK_RANGE(ctk_multisample->fsaa_scale);
+    GtkWidget *fsaa_menu = ctk_multisample->fsaa_menu;
+    GtkWidget *fxaa_checkbox = ctk_multisample->fxaa_enable_check_button;
+    gboolean fxaa_value;
+
+    /* The FSAA dropdown menu is: 0 == app, 1 == override, 2 == enhance */
+    gint fsaa_idx = CTK_DROP_DOWN_MENU(fsaa_menu)->current_selected_item;
+    
+    if (fsaa_value != NV_CTRL_FSAA_MODE_NONE) {
+        g_signal_handlers_block_by_func(G_OBJECT(fxaa_checkbox),
+                                        G_CALLBACK(fxaa_checkbox_toggled),
+                                        (gpointer) ctk_multisample);
+
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fxaa_checkbox), FALSE);
+
+        g_signal_handlers_unblock_by_func(G_OBJECT(fxaa_checkbox),
+                                          G_CALLBACK(fxaa_checkbox_toggled),
+                                          (gpointer) ctk_multisample);
+    }
+
+    fxaa_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fxaa_checkbox));
+
+    gtk_widget_set_sensitive(GTK_WIDGET(fsaa_range), 
+                             (fsaa_idx != 0) && // not app controlled
+                             (fxaa_value == NV_CTRL_FXAA_DISABLE));
+    gtk_widget_set_sensitive(GTK_WIDGET(fxaa_checkbox),
+                             fsaa_value == NV_CTRL_FSAA_MODE_NONE);
+} /* update_fxaa_from_fsaa_change() */
+
+/*
+ * update_fsaa_from_fxaa_change - helper function for changes to fxaa in order
+ * to update fsaa and enable/disable fxaa or fsaa widgets based on the new 
+ * value of fxaa.
+ */
+
+static void update_fsaa_from_fxaa_change (CtkMultisample *ctk_multisample,
+                                          gboolean fxaa_enabled)
+{
+    GtkWidget *fxaa_checkbox = ctk_multisample->fxaa_enable_check_button;
+    GtkRange *fsaa_range = GTK_RANGE(ctk_multisample->fsaa_scale);
+    gint fsaa_value_none = map_nv_ctrl_fsaa_value_to_slider(ctk_multisample, 0);
+    GtkWidget *fsaa_menu = ctk_multisample->fsaa_menu;
+
+    /* The FSAA dropdown menu is: 0 == app, 1 == override, 2 == enhance */
+    gint fsaa_idx = CTK_DROP_DOWN_MENU(fsaa_menu)->current_selected_item;
+    
+    gint fsaa_val;
+
+    if (fxaa_enabled == NV_CTRL_FXAA_ENABLE) {
+        g_signal_handlers_block_by_func(G_OBJECT(fsaa_range),
+                                        G_CALLBACK(fsaa_value_changed),
+                                        (gpointer) ctk_multisample);
+
+        gtk_range_set_value(fsaa_range, fsaa_value_none);
+
+        g_signal_handlers_unblock_by_func(G_OBJECT(fsaa_range),
+                                          G_CALLBACK(fsaa_value_changed),
+                                          (gpointer) ctk_multisample);
+    }
+
+    fsaa_val = gtk_range_get_value(fsaa_range);
+    if (fsaa_val > NV_CTRL_FSAA_MODE_MAX) fsaa_val = NV_CTRL_FSAA_MODE_MAX;
+    if (fsaa_val < 0) fsaa_val = 0;
+    fsaa_val = ctk_multisample->fsaa_translation_table[fsaa_val];
+
+    gtk_widget_set_sensitive(GTK_WIDGET(fxaa_checkbox), 
+                             (fxaa_enabled == NV_CTRL_FXAA_ENABLE) || 
+                             (fsaa_val == NV_CTRL_FSAA_MODE_NONE));
+    gtk_widget_set_sensitive(GTK_WIDGET(fsaa_range), 
+                             (fsaa_idx != 0) && // not app controlled
+                             (fxaa_enabled == NV_CTRL_FXAA_DISABLE));
+} /* update_fsaa_from_fxaa_change */
 
 /*
  * fsaa_value_changed() - callback for the "value-changed" signal from
@@ -923,12 +1090,81 @@ static void fsaa_value_changed(GtkRange *range, gpointer user_data)
     val = ctk_multisample->fsaa_translation_table[val];
 
     NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_FSAA_MODE, val);
+
+    update_fxaa_from_fsaa_change(ctk_multisample, val);
     
     post_fsaa_value_changed(ctk_multisample, val);
 
 } /* fsaa_value_changed() */
 
 
+/*
+ * fxaa_checkbox_toggled - callback for a change to
+ * the FXAA settings in the control panel
+ */
+static void fxaa_checkbox_toggled(GtkWidget *widget,
+                                  gpointer user_data)
+{
+    CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    gboolean enabled;
+
+    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+    NvCtrlSetAttribute(ctk_multisample->handle, NV_CTRL_FXAA, enabled);
+
+    update_fsaa_from_fxaa_change(ctk_multisample, enabled);
+
+    post_fxaa_toggled(ctk_multisample, enabled);
+
+} /* fxaa_checkbox_toggled */
+
+
+/*
+ * fxaa_update_received() - callback function for when the
+ * NV_CTRL_FXAA attribute is changed by another NV-CONTROL
+ * client.
+ */
+
+static void fxaa_update_received(GtkObject *object,
+                                 gpointer arg1, gpointer user_data)
+{
+    CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
+    CtkMultisample *ctk_multisample = CTK_MULTISAMPLE(user_data);
+    gboolean fxaa_value = event_struct->value;
+    GtkWidget *check_button = ctk_multisample->fxaa_enable_check_button;
+
+    g_signal_handlers_block_by_func(G_OBJECT(check_button),
+                                    G_CALLBACK(fxaa_checkbox_toggled),
+                                    (gpointer) ctk_multisample);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), fxaa_value);
+    
+    update_fsaa_from_fxaa_change(ctk_multisample, fxaa_value);
+
+    g_signal_handlers_unblock_by_func(G_OBJECT(check_button),
+                                      G_CALLBACK(fxaa_checkbox_toggled),
+                                      (gpointer) ctk_multisample);
+
+    post_fxaa_toggled(ctk_multisample, fxaa_value);
+
+} /* fxaa_update_received() */
+
+/*
+ * post_fxaa_toggled() - helper function for fxaa_button_toggled() 
+ * and fxaa_update_received(); this does whatever work is necessary 
+ * after the app control check button has been toggled.
+ */
+
+static void
+post_fxaa_toggled(CtkMultisample *ctk_multisample, gboolean enable)
+{
+    ctk_config_statusbar_message(ctk_multisample->ctk_config,
+                                 "FXAA "
+                                 "%s.", enable ? "enabled" : "disabled");
+
+} /* post_fxaa_toggled() */
+
+
 
 /*
  * fsaa_update_received() - callback function for when the
@@ -956,6 +1192,8 @@ static void fsaa_update_received(GtkObject *object,
                                     (gpointer) ctk_multisample);
     
     gtk_range_set_value(range, val);
+
+    update_fxaa_from_fsaa_change(ctk_multisample, event_struct->value);
     
     g_signal_handlers_unblock_by_func(G_OBJECT(range),
                                       G_CALLBACK(fsaa_value_changed),
@@ -1278,7 +1516,7 @@ GtkTextBuffer *ctk_multisample_create_help(GtkTextTagTable *table,
                       "rendering may decrease performance.");
 
         ctk_help_para(b, &i, "You can also configure Antialiasing "
-                      "using the __GL_FSAA_MODE environment varible (see "
+                      "using the __GL_FSAA_MODE environment variable (see "
                       "the README for details).  The __GL_FSAA_MODE "
                       "environment variable overrides the value in "
                       "nvidia-settings.");
@@ -1407,6 +1645,11 @@ GtkTextBuffer *ctk_multisample_create_help(GtkTextTagTable *table,
         }
     }
 
+    if (ctk_multisample->active_attributes & __FXAA) {
+        ctk_help_term(b, &i, "Enable FXAA");
+        ctk_help_para(b, &i, __fxaa_enable_help);
+    }
+
     if (ctk_multisample->active_attributes & __LOG_ANISO_RANGE) {
         ctk_help_heading(b, &i, "Anisotropic Filtering");
         
@@ -1419,7 +1662,7 @@ GtkTextBuffer *ctk_multisample_create_help(GtkTextTagTable *table,
                       "expense of some performance.");
 
         ctk_help_para(b, &i, "You can also configure Anisotropic filtering "
-                      "using the __GL_LOG_MAX_ANISO environment varible "
+                      "using the __GL_LOG_MAX_ANISO environment variable "
                       "(see the README for details).  The "
                       "__GL_LOG_MAX_ANISO environment variable overrides "
                       "the value in nvidia-settings.");
